#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "hd00Contents.h"
#include "hd00Midi.h"

#define FILENAME "songtmp"
char	*HotDog00, *AddrFName;

int	main( int argc, char *argv[])
{
  unsigned int i, j, k, TableNo, SongNo;
  unsigned long songEntryTable[TOC_LENGTH];
  unsigned int noSongEntryTables;
  SongEntry songEntry[TOC_LENGTH];
  unsigned int noSongs, noSongsTable;
  unsigned long offset;
  int length, ch, keyLength, argError, mode, Language;
  char filename[64];
  FILE *f;
  FILE *f2;
  FILE *outFile;
  FILE *lFile;
  FILE *mFile;

  argError = 1;
  i = 1;
  mode = ASCII_MODE; //default
  if (argc==4 && strlen(argv[1])==2)
  {
    i++;
    if (argv[1][0] == '-')
    {
      if (argv[1][1] == 'a')
      {
        mode = ASCII_MODE;
        Language= LNG_ENGLISH;
        argError = 0;
      }
      else if (argv[1][1] == 'd')
      {
        mode = DUALBYTE_MODE;
        Language= LNG_CHINESE;
        argError = 0;
      }
    }
  }
  else if( argc==3)
  {
    argError = 0;
  }

  if(argError)
  {
    printf("Usage: %s [-a|d] <hotdog00> <song ID (hex)>", argv[0]);
    return 0;
  }

  HotDog00= argv[i];
  i++;
  sscanf( argv[i],"%x", &offset);
  TableNo= offset >> 8;
  SongNo=(unsigned char)offset& 0xff;
  printf("%02x %02x\n", TableNo, SongNo);

  f = fopen( HotDog00, "rb");

  noSongEntryTables = getTableOfContents(f, songEntryTable);

  noSongsTable = getSongEntryTable(f, songEntryTable[TableNo], songEntry);

  if (songEntry[SongNo].length != 0)
  {
    /* Dump the whole song entry (header+lyrics+midi) to *.dat file */
    offset = songEntry[SongNo].offset;
    length = songEntry[SongNo].length;
    sprintf(filename, "%s.dat", FILENAME);
    outFile= fopen(filename, "wb");
    fseek(f, offset, SEEK_SET);
    for (i=0; i<length; i++)
    {
      ch = getc(f);
      if (ch == EOF)
      {
        break;
      }
      putc(ch, outFile);
    }
    fclose(outFile);

    /* Decode and dump the lyrics part of the song entry to a *.lpd file (Lyrics Part Data)*/
    offset = songEntry[SongNo].offset+8;
    length = songEntry[SongNo].midiOffset+4;
    sprintf(filename, "%s.lpd", FILENAME);
    outFile= fopen(filename, "wb");
    keyLength = ReadLyricDat(f, offset, outFile, length, mode);
    fclose(outFile);

    /* Exract ASCII from lyrics part and dump to *.txt file*/
    /*
    f2 = fopen(filename, "rb");
    sprintf(filename, "%s.txt", FILENAME);
    outFile= fopen(filename, "w");
    if (mode==ASCII_MODE)
    {
      mapLyric(f2, outFile, keyLength);
    }
    else if (mode==DUALBYTE_MODE)
    {
      mapLyric2b(f2, outFile, keyLength);
    }
    fclose(outFile);
    fclose(f2);
    */
    /* Dump the midi part of the song entry to a *.mpd file (Midi Part Data) */
    offset = songEntry[SongNo].offset + songEntry[SongNo].midiOffset + 12;
    length = songEntry[SongNo].length - songEntry[SongNo].midiOffset-15+3;
    sprintf(filename, "%s.mpd", FILENAME);
    outFile= fopen(filename, "wb");
    fseek(f, offset, SEEK_SET);
    for (i=0; i<length; i++)
    {
      ch = getc(f);
      if (ch == EOF)
      {
        break;
      }
      putc(ch, outFile);
    }
    fclose(outFile);

    /* Extract midi part and convert to *.mid file */
    offset =  songEntry[SongNo].offset + songEntry[SongNo].midiOffset + 15;
    length =  songEntry[SongNo].length - songEntry[SongNo].midiOffset - 15;
//      length -= length % 5;

    /* Convert *.lpd + *.mpd file back to *.dat file */
    /*
    sprintf(filename, "%02x_%02x.d2", TableNo, SongNo);
    outFile= fopen( filename, "wb");
    sprintf(filename, "%02x_%02x.lpd", TableNo, SongNo);
    lFile = fopen(filename, "rb");
    sprintf(filename, "%02x_%02x.mpd", TableNo, SongNo);
    mFile = fopen(filename, "rb");
    writeDat(lFile, mFile, outFile, mode);
    fclose(mFile);
    fclose(lFile);
    fclose(outFile);
*/
    sprintf(filename, "%s.lpd", FILENAME);
    lFile = fopen(filename, "rb");

    sprintf(filename, "%s.mid", FILENAME);
    outFile= fopen( filename, "wb");
    long LyrPtr= FindLyrStart( lFile, Language);
//    printf("Lyric start: %04x\n", LyrPtr);
    ReadMidiDat(f, offset, outFile, length/5, lFile, Language);

    fclose(outFile);
    fclose(lFile);
  }
  fclose( f );

  return 0;
}
