#include <stdio.h>
#include <ctype.h>

#include "hd00Contents.h"


int key2Ascii[] = {
  0x23, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x31, 0x00, 0x00, 0x00,

  0x0d, 0x0a, 0x00, 0x00,
  0x40, 0x00, 0x00, 0x00,
  0x40, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,

  0x40, 0x00, 0x00, 0x00,
  0x31, 0x00, 0x00, 0x00,
  0x32, 0x00, 0x00, 0x00,
  0x0d, 0x0a, 0x00, 0x00,
  0x40, 0x00, 0x00, 0x00
};

int key2Dual[] = {
  0x23, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x31, 0x00, 0x00, 0x00,

  0x0d, 0x0a, 0x00, 0x00,
  0x40, 0x00, 0x00, 0x00,
  0x40, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,

  0x40, 0x00, 0x00, 0x00,
//  0x31, 0x00, 0x00, 0x00,
  0x30, 0x00, 0x00, 0x00,
  0x32, 0x00, 0x00, 0x00,
  0x0d, 0x0a, 0x00, 0x00,
  0x40, 0x00, 0x00, 0x00
};

unsigned long get32(FILE *f)
{
  unsigned long tmp, tmp2;

  tmp = 0;

  tmp2 = getc(f);
  tmp = (tmp << 8) | (tmp2 & 0x0FF);
  tmp2 = getc(f);
  tmp = (tmp << 8) | (tmp2 & 0x0FF);
  tmp2 = getc(f);
  tmp = (tmp << 8) | (tmp2 & 0x0FF);
  tmp2 = getc(f);
  tmp = (tmp << 8) | (tmp2 & 0x0FF);

  return tmp;
}

void put32(FILE *f, unsigned long tmp)
{
  int tmp2;

  tmp2 = (tmp >> 24) & 0x0FF;
  putc(tmp2, f);
  tmp2 = (tmp >> 16) & 0x0FF;
  putc(tmp2, f);
  tmp2 = (tmp >> 8) & 0x0FF;
  putc(tmp2, f);
  tmp2 = (tmp & 0x0FF);
  putc(tmp2, f);
}

int getTableOfContents(FILE *inFile, unsigned long toc[TOC_LENGTH])
{
  int i;
  unsigned int tmp, tmp2;

  fseek(inFile, TOC_OFFSET, SEEK_SET);

  /*Read Table of Contents*/
  for (i=0; i<TOC_LENGTH; i++)
  {
    toc[i] = get32(inFile);
  }

  /*printf("Table of song entry tables:\n");*/

  /*Verify if addresses in table are valid*/
  /*TODO: Compare with length of file instead.*/
  for(i=0; i<TOC_LENGTH; i++)
  {
    tmp = fseek(inFile, toc[i], SEEK_SET);
    tmp2 = getc(inFile);

    /*printf("%4d: %8X %d %X\n", i, songEntryTable[i], tmp, tmp2);*/

    if(tmp != 0 | tmp2 == EOF)
    {
      break;
    }
  }

  /*printf("noSongEntryTables: %d\n\n", i);*/
  return i;
}


int getSongEntryTable(FILE *inFile, unsigned long offset, SongEntry songEntry[TOC_LENGTH])

{
  int noSongs, i;
  unsigned int tmp, tmp2, tmp3, tmp4;

  fseek(inFile, offset, SEEK_SET);
    
  for (i=0; i<TOC_LENGTH; i++)
  {
    songEntry[i].offset = get32(inFile);
    songEntry[i].length = 0;
    songEntry[i].midiOffset = 0;
  }
  for (i=0; i<TOC_LENGTH; i++)
  {
    songEntry[i].unknown = get32(inFile);
  }
  
  /*printf("CONTENT: %8X - %8X: Table of song entries %d\n", songEntryTable[j], songEntryTable[j]+0x800, j);
    printf("Table of song entries %d:\n", j);*/

  noSongs = 0;
  for(i=0; i<TOC_LENGTH; i++)
  {
    tmp = fseek(inFile, songEntry[i].offset, SEEK_SET);
    tmp2 = getc(inFile);

    /*printf("%4d: %8X ", i, songEntry[i]);*/

    if(tmp != 0 | tmp2 == EOF)
    {
      /*printf("\n");*/
      continue;
    }
    else
    {
      fseek(inFile, songEntry[i].offset, SEEK_SET);
      tmp = get32(inFile);
      tmp2 = get32(inFile);

      /* Check if this is a song entry */
      if ((tmp < 0x1FFFF) && (tmp > 0x100) && (tmp > tmp2) && (tmp2 < 0x3FFF) && (tmp2 > 0x10) && (songEntry[i].unknown != 0))
      {
	fseek(inFile, songEntry[i].offset+tmp2+14, SEEK_SET);
            
	tmp3 = getc(inFile);
	tmp4 = getc(inFile);
	/* One more check if this is a song entry ("00 00" in beginning of midi part) */
	if ((tmp3 == 0) || (tmp4 == 0))
        {
	  noSongs++;
	  songEntry[i].length = tmp;
	  songEntry[i].midiOffset = tmp2;
	}
	else
        {
	  /*printf("Not a song entry!\n");*/
	  continue;
	}
      }
      else
      {
	/*printf("Not a song entry.\n");*/
	continue;
      }
    }
  }
  return noSongs;
}

#ifdef __BORLANDC__
int ReadLyricDat(FILE *inFile, int offset, FILE *outFile, int length, int mode)
#else
int ReadLyricDat(FILE *inFile, int offset, FILE *outFile, int length, int mode=ASCII_MODE)
#endif
{
  int n, k, k2, tmp;
  int keyLen;
  int key[100];
  int *key2;

  if(mode==ASCII_MODE)
  {
    key2 = key2Ascii;
  }
  else if (mode==DUALBYTE_MODE)
  {
    key2 = key2Dual;
  }

  /* Write the first 4 bytes to output file */
  fseek(inFile, offset, SEEK_SET);
  for (n=0; n<4; n++)
  {
    tmp = getc(inFile);
    if (tmp == EOF)
    {
      return 0;
    }
    putc(tmp, outFile);
  }

  /* Determine the Key length */
  for(keyLen=0; keyLen<100; keyLen++)
  {
    key[keyLen] = getc(inFile);
    if ((keyLen>3) &&
        (key[keyLen-2]==key[1]) &&
        (key[keyLen-1]==key[2]) &&
        (key[keyLen]==key[3]))
    {
      keyLen -= 3;
      printf("Key length: %d\n", keyLen);
      break;
    }
  }
  if (keyLen>60)
  {
    printf("First attempt to determine Key length failed\n");
    for(keyLen=8; keyLen<100; keyLen++)
    {
      if((key[keyLen-2]==key[5]) &&
         (key[keyLen-1]==key[6]) &&
         (key[keyLen]==key[7]))
      {
        keyLen -= 7;
        printf("Key length: %d\n", keyLen);
        break;
      }
    }
  }
  if (keyLen>60)
  {
    printf("Failed to determine Key length\n");
    return 0;
  }

  /* Write key to output file */
  for(k=0; k<keyLen; k++)
  {
    putc(key[k], outFile);
  }

  /* Decode and write lyrics part to output file */
  fseek(inFile, offset+keyLen+4, SEEK_SET);
  n = keyLen+4;
  k2 = 0;
  while (n < length)
  {
    int c0,c1,c2,c3;
    for (k=0; k<keyLen; k++)
    {
      tmp = getc(inFile);
      /* === For debugging ====
      switch (k%4)
      {   case 0:
              c0= tmp ^key[k] ^key2[k];
              printf("%3d k=0x%02x k2=0x%02x v=%02x o=%02x\n", k,key[k],key2[k],tmp, c0);
              break;

          case 1:
              c1= tmp ^key[k] ^key2[k];
              printf("%3d k=0x%02x k2=0x%02x v=%02x o=%02x\n", k,key[k],key2[k],tmp, c1);
              break;

          case 2:
              c2= tmp ^key[k] ^key2[k];
              printf("%3d k=0x%02x k2=0x%02x v=%02x o=%02x\n", k,key[k],key2[k],tmp, c2);
              break;

          case 3:
              c3= tmp ^key[k] ^key2[k];
              printf("%3d k=0x%02x k2=0x%02x v=%02x o=%02x\t[%c] %02x %02x\n", k,key[k],key2[k],tmp, c0,(char)c0, c3,c2);
              break;
      }
      */
      tmp = tmp ^key[k] ^key2[k];
/*
      if ((k%4) == 0)
      {
        tmp = (tmp ^ key2[k2]);
        k2 = (k2+1) % (keyLen/4);
      }
*/
      putc(tmp, outFile);
      n++;
      if (n>=length)
      {
        break;
      }
    }
  }
  return keyLen;
}


unsigned int findMidi(FILE *f)
{
  unsigned int i;
  unsigned int prev[3];
  
  
  prev[0] = 0xCC;
  prev[1] = 0xCC;
  prev[2] = 0xCC;
  printf("; ");
  for (i=0; i < 50000; i++)
  {
    prev[0] = prev[1];
    prev[1] = prev[2];
    prev[2] = getc(f);

    if (i<18)
    {
      printf("%2X ", prev[2]);
    }
    
    if (prev[0] == 0 && prev[1] == 0xFF && (prev[2] & 0x0F0) == 0xC0)
    {
      printf("- %4X %4d", i-2, i-2);
      return 1;
    }
  }
  return 0;
}


void findLyric(FILE *f, unsigned int len)
{
  unsigned int i, ch, nCh;

  for (i=0; i<len; i++)
  {
    ch = getc(f);
    if (isalpha(ch))
    {
      printf("%c", ch);
      nCh++;
      if(nCh > 40)
      {
        printf(" %4d %4d", i,len);
        return;
      }
    }
  }
}




void makeMap(FILE *f)
{
  int i, n, ch, ch2, nProblem;
  unsigned long tmp;
  int chMap[265];
  int chM0a[256];
  int problem[256];
  char lyrics[] = "Oooh I need your love babe Guess you know it's true Hope you need my love babe Just like I need you Hold me love me Hold me love me I ain't got nothin' but love babe Eight days a week Love you everyday girl Always on my mind One thing I Can say girl Love you all the time Hold me love me Hold me love me I ain't got nothin' but love babe Eight days a week Eight days a week I love you Eight days a week is not enough to show I care Oooh I need your love babe Guess you know it's true Hope you need my love babe Just like I need you Hold me love me Hold me love me I ain't got nothin' but love babe Eight days a week Eight days a week I love you Eight days a week is not enough to show I care Love you everyday girl Always on my mind One thing I can say girl Love you all the time Hold me love me Hold me love me I ain't got nothin' but love babe Eight days a week Eight days a week Eight days a week";

  // Reset character mapping
  for(i=0; i<256; i++)
  {
    chMap[i] = 0xFFFF;
    chM0a[i] = 0xFFFF;
    problem[i]=0;
  }

  /* Skip header part of song */
  for(i=0;; i++)
  {
    tmp = get32(f);

    if ((tmp & 0x0000FFFF) != 0)
    {
      fseek(f, -4, SEEK_CUR);
      break;
    }
  }
  
  n=0;
  nProblem=0;
  for(i=0;; i++)
  {
    ch = getc(f);
    if (ch == EOF || lyrics[n] == '\0')
    {
      break;
    }

    ch2 = getc(f);
    if (ch2 == EOF)
    {
      break;
    }

    if (ch2 == 0)
    {
      ch = (ch & 0x00FF);
      if(lyrics[n] == '\'')
      {
        printf("  chMap[0x%02x] = \'\\\'\';", ch);
      }
      else
      {
        printf("  chMap[0x%02x] = \'%c\';", ch, lyrics[n]);
      }

      if(chMap[ch] == 0xFFFF)
      {
        chMap[ch] = lyrics[n];
      }
      else
      {
        if (chMap[ch] == lyrics[n])
        {
          printf(" // dup");
        }
        else
        {
          chMap[ch] = lyrics[n];
          nProblem++;
          printf(" // problem %d", nProblem);
          problem[ch]++;
        }
      }
    }
    else if (ch2 == 0x0a)
    {

      ch = (ch & 0x00FF);
      if(lyrics[n] == '\'')
      {
        printf("  chM0a[0x%02x] = \'\\\'\';", ch);
      }
      else
      {
        printf("  chM0a[0x%02x] = \'%c\';", ch, lyrics[n]);
      }

      if(chM0a[ch] == 0xFFFF)
      {
        chM0a[ch] = lyrics[n];
      }
      else
      {
        if (chM0a[ch] == lyrics[n])
        {
          printf(" // dup");
        }
        else
        {
          chM0a[ch] = lyrics[n];
          nProblem++;
          printf(" // problem %d", nProblem);
          problem[ch]++;
        }
      }
      printf(" // 0x0a");
    }
    else
    {
      printf("Error\n");
    }
    
    ch = getc(f);
    if (ch == EOF)
    {
      break;
    }
    else if (ch == 0)
    {
      ch = getc(f);
      if (ch == EOF)
      {
        break;
      }
      else if (ch == 0)
      {
        printf(" // 0x0000");
      }
    }
    else
    {
      ch = getc(f);
      if (ch == EOF)
      {
        break;
      }      
    }

    printf("\n");
    n++;  
  }

  printf("\n");

  for(i=0; i<256; i++)
  {
    if(chMap[i] == 0xFFFF)
    {
      printf(".");
    }
    else
    {
      printf("%c", chMap[i]);
    }
  }
  printf("\n");
  
  for(i=0; i<256; i++)
  {
    if(chM0a[i] == 0xFFFF)
    {
      printf(".");
    }
    else
    {
      printf("%c", chM0a[i]);
    }
  }
  printf("\n\n");

//    for(i='a',n=33;i<'z';i++,n++)
//    {
//      chMap[n]=i;
//    }
//    for(i='a',n=81;i<'z';i++,n++)
//    {
//      if(chMap[n]!=0xFFFF && chMap[n]!=i)
//      {
//        printf("%d: %c != %c\n", n, chMap[n], i);
//      }
//      chMap[n]=i;
//    }
//    for(i=0; i<256; i++)
//    {
//      if(chMap[i] == 0xFFFF)
//      {
//        printf(".");
//      }
//      else
//      {
//        printf("%c", chMap[i]);
//      }
//    }

  printf("\n\n");

  for(i=0; i<256; i++)
  {
    if(chMap[i] != 0xFFFF)
    {
      if(chMap[i] == '\'')
      {
        printf("  chMap[0x%02x] = \'\\\'\';\n", i);
      }
      else
      {
        printf("  chMap[0x%02x] = \'%c\';\n", i, chMap[i]);
      }
    }
  }
  for(i=0; i<256; i++)
  {
    if(chM0a[i] != 0xFFFF)
    {
      if(chM0a[i] == '\'')
      {
        printf("  chM0a[0x%02x] = \'\\\'\';\n", i);
      }
      else
      {
        printf("  chM0a[0x%02x] = \'%c\';\n", i, chM0a[i]);
      }
    }
  }

  for(i=0; i<256; i++)
  {
    if(problem[i] != 0)
    {
      printf("%03d %02x: %d\n", i,i, problem[i]);
    }
  }
}

void mapLyricOld2(FILE *f, FILE *outFile)
{
  int i, ch, ch2, n;
  int chMap[265];
  int chM0a[265];

  // Reset character mapping
  for(i=0; i<256; i++)
  {
    chMap[i] = '.';
    chM0a[i] = '.';
  }

  chMap[0x00] = ' ';
  chMap[0x01] = 'A';
  chMap[0x03] = ' ';
  chMap[0x04] = '\'';
  chMap[0x05] = 'E';
  chMap[0x08] = 'H';
  chMap[0x09] = 'I';
  chMap[0x0c] = 'L';
  chMap[0x10] = ' ';
  chMap[0x11] = ' ';
  chMap[0x12] = ' ';
  chMap[0x15] = '\'';
  chMap[0x16] = '\'';
  chMap[0x17] = '\'';
  chMap[0x21] = 'a';
  chMap[0x22] = 'b';
  chMap[0x23] = 'c';
  chMap[0x24] = 'd';
  chMap[0x25] = 'e';
  chMap[0x27] = 'g';
  chMap[0x28] = 'h';
  chMap[0x29] = 'i';
  chMap[0x2b] = 'k';
  chMap[0x2c] = 'l';
  chMap[0x2d] = 'm';
  chMap[0x2e] = 'n';
  chMap[0x2f] = 'o';
  chMap[0x30] = 'p';
  chMap[0x32] = 'r';
  chMap[0x33] = 's';
  chMap[0x34] = 't';
  chMap[0x35] = 'u';
  chMap[0x36] = 'v';
  chMap[0x37] = 'w';
  chMap[0x39] = 'y';
  chMap[0x40] = 'r';
  chMap[0x41] = 's';
  chMap[0x42] = 'a';
  chMap[0x43] = 's';
  chMap[0x44] = 't';
  chMap[0x45] = 'u';
  chMap[0x46] = 'w';
  chMap[0x47] = 'd';
  chMap[0x48] = 'y';
  chMap[0x49] = 'y';
  chMap[0x4a] = 'i';
  chMap[0x4b] = 'h';
  chMap[0x4c] = 'o';
  chMap[0x4e] = 'm';
  chMap[0x4f] = 'l';
  chMap[0x50] = 'a';
  chMap[0x51] = 'a';
  chMap[0x52] = 'b';
  chMap[0x53] = 'a';
  chMap[0x54] = 'e';
  chMap[0x55] = 'e';
  chMap[0x56] = 'g';
  chMap[0x57] = 'e';
  chMap[0x58] = 'h';
  chMap[0x59] = 'i';
  chMap[0x5a] = 'h';
  chMap[0x5b] = 'k';
  chMap[0x5c] = 'l';
  chMap[0x5d] = 'o';
  chMap[0x5e] = 'o';
  chMap[0x5f] = 'n';
  chMap[0x60] = ' ';
  chMap[0x66] = 'E';
  chMap[0x67] = '\'';
  chMap[0x6b] = 'H';
  chMap[0x6c] = 'O';
  chMap[0x73] = 'A';
  chMap[0x74] = 'E';
  chMap[0x75] = 'E';
  chMap[0x77] = 'E';
  chMap[0x78] = 'H';
  chMap[0x79] = 'I';
  chMap[0x7a] = 'J';
  chMap[0x7b] = 'I';
  chMap[0x7c] = 'L';
  chMap[0x7d] = 'O';
  chM0a[0x2a] = '\'';
  chM0a[0x2d] = ' ';
  chM0a[0x2e] = ' ';
  chM0a[0x3c] = ' ';
  chM0a[0x3d] = ' ';
  chM0a[0x3f] = ' ';
  chM0a[0x42] = 'O';
  chM0a[0x44] = 'I';
  chM0a[0x48] = 'E';
  chM0a[0x4d] = ' ';
  chM0a[0x60] = 'm';
  chM0a[0x61] = 'l';
  chM0a[0x62] = 'o';
  chM0a[0x63] = 'n';
  chM0a[0x64] = 'i';
  chM0a[0x65] = 'h';
  chM0a[0x66] = 'k';
  chM0a[0x68] = 'e';
  chM0a[0x69] = 'd';
  chM0a[0x6a] = 'g';
  chM0a[0x6c] = 'a';
  chM0a[0x6f] = 'b';
  chM0a[0x74] = 'y';
  chM0a[0x78] = 'u';
  chM0a[0x79] = 't';
  chM0a[0x7a] = 'w';
  chM0a[0x7b] = 'v';
  chM0a[0x7e] = 's';
  chM0a[0x7f] = 'r';

//    for(i='a',n=33;i<'z';i++,n++)
//    {
//      chMap[n]=i;
//    }
//    for(i='a',n=81;i<'z';i++,n++)
//    {
//      chMap[n]=i;
//    } 
  for(int i=0;; i++)
  {
    ch = getc(f);

    if (ch == EOF)
    {
      break;
    }
    
    if ((i % 4) == 0)
    {
      ch2 = getc(f);
      if (ch2 == EOF)
      {
        break;
      }
      i++;

      ch = (ch & 0x00FF);
      if (ch2 == 0)
      {
        printf("%c", chMap[ch]);
        fprintf(outFile,"%c", chMap[ch]);
      }
      else if (ch2 == 0x0a)
      {
        printf("%c", chM0a[ch]);
        fprintf(outFile,"%c", chM0a[ch]);        
      }
      else
      {
        printf("?");
      }
    }
  }
  printf("\n");
}

void mapLyric(FILE *f, FILE *outFile, int keyLength)
{
  int i, ch;

  /* First 2 letters of titel*/
  ch = getc(f);
  printf("%02x", ch);
  fprintf(outFile,"%02x", ch);
  ch = getc(f);
  printf(" %c", ch);
  fprintf(outFile," %c", ch);
  ch = getc(f);
  printf(" %02x", ch);
  fprintf(outFile," %02x", ch);
  ch = getc(f);
  printf(" %c\n", ch);
  fprintf(outFile," %c\n", ch);

  printf("Key length: %d\n", keyLength);
  fprintf(outFile,"Key length: %d\n", keyLength);

  printf("Key:");
  fprintf(outFile,"Key:");
  for(i=0; i<keyLength; i++)
  {
    ch = getc(f);
    printf(" %02x", ch);
    fprintf(outFile," %02x", ch);
  }

  for(i=0;; i++)
  {
    ch = getc(f);

    if (ch == EOF)
    {
      break;
    }

    if ((i % 4) == 0)
    {
      ch = (ch & 0x00FF);
      if (ch==0x0d)
      {
        printf("\n");
      }
      else if (isprint(ch))
      {
        printf("%c", ch);
        fprintf(outFile,"%c", ch);
      }
      else
      {
        printf("[%x]", ch);
        fprintf(outFile,"[%x]",ch);
      }
    }
  }
  printf("\n");
}

void mapLyric2b(FILE *f, FILE *outFile, int keyLength)
{
  int i, ch0, ch;

  /* First 2 letters of titel*/
  ch = getc(f);
  printf("%02x", ch);
  fprintf(outFile,"%02x", ch);
  ch = getc(f);
  printf(" %c", ch);
  fprintf(outFile," %c", ch);
  ch = getc(f);
  printf(" %02x", ch);
  fprintf(outFile," %02x", ch);
  ch = getc(f);
  printf(" %c\n", ch);
  fprintf(outFile," %c\n", ch);

  printf("Key length: %d\n", keyLength);
  fprintf(outFile,"Key length: %d\n", keyLength);

  printf("Key:");
  fprintf(outFile,"Key:");
  for(i=0; i<keyLength; i++)
  {
    ch = getc(f);
    printf(" %02x", ch);
    fprintf(outFile," %02x", ch);
  }

  for(i=0;; i++)
  {
    ch = getc(f);

    if (ch == EOF)
    {
      break;
    }

    if ((i % 4) == 0 || (i % 4) == 1)
    {
      ch = (ch & 0x00FF);
      if (ch==0x0d)
        fprintf(outFile,"\n");
      else if (ch)
        fprintf(outFile,"%c", ch);
    }
/*
    if ((i % 4) == 0)
      ch0 = (ch & 0x00FF);

    if ((i % 4) == 1)
    {
      ch = (ch & 0x00FF);
	if( ch0)	// Dual byte
        fprintf(outFile,"%c%c", ch0,ch);
      else if (ch==0x0d)
        fprintf(outFile,"\n");
      else
        fprintf(outFile,"%c", ch0);
    }
*/
  }
  printf("\n");
}

#ifdef __BORLANDC__
void writeDat(FILE *lyricsFile, FILE *midiFile, FILE *outFile, int mode)
#else
void writeDat(FILE *lyricsFile, FILE *midiFile, FILE *outFile, int mode=ASCII_MODE)
#endif
{
  int i, tmp, keyLen, k, done;
  unsigned long tmpLong, len, len2;
  int key[100];
  int *key2;

  if (mode==ASCII_MODE)
  {
    key2 = key2Ascii;
  }
  else if (mode==DUALBYTE_MODE)
  {
    key2 = key2Dual;
  }


  /* Reserve space for length and offset */
  put32(outFile, 0);
  put32(outFile, 0);
  len = 8;

  /* Write first 4 bytes from lyrics file (first 2 letters of Song Title)*/
  tmpLong = get32(lyricsFile);
  put32(outFile, tmpLong);
  len += 4;

  /* Write key */
  for (keyLen=0;;keyLen+=4)
  {
    key[keyLen] = getc(lyricsFile);
    key[keyLen+1] = getc(lyricsFile);
    key[keyLen+2] = getc(lyricsFile);
    key[keyLen+3] = getc(lyricsFile);
    if (key[keyLen+2] == 0 && key[keyLen+3] == 0)
    {
      break;
    }
    putc(key[keyLen], outFile);
    putc(key[keyLen+1], outFile);
    putc(key[keyLen+2], outFile);
    putc(key[keyLen+3], outFile);
    len += 4;
  }
  
  /* Encode and write lyrics file */
  fseek(lyricsFile, keyLen+4, SEEK_SET);
  done = 0;
  while(!done)
  {
    for (k=0; k<keyLen; k++)
    {
      tmp = getc(lyricsFile);
      if(tmp == EOF)
      {
        done = 1;
        break;
      }
      tmp = tmp ^ key[k] ^ key2[k];
      putc(tmp, outFile);
      len++;
    }
  }
  /* Save the midiOffset*/
  len2=len-12;

  /* Write midi part from midi file */
  for (;;)
  {
    tmp = getc(midiFile);
    if(tmp == EOF)
    {
      break;
    }
    putc(tmp, outFile);
    len++;
  }

  fseek(outFile, 0, SEEK_SET);
  put32(outFile, len);
  put32(outFile, len2);
}
