/***************************************************************************/
// Generates a song list from HOTDOG20.DAT into SQLite DB
//---------------------------------------------------------------------------
// Author: Phuoc Can HUA (09/09/07)
/***************************************************************************/

#include <fstream.h>
#include <stdio.h>
//#include <ctype.h>
#include <map>

#include "dbtoc.h"
#include "hotdog00.h"


int	SongTotal= 0, TocOffset;

typedef struct
{		int	SongTotal, Offset;
} InputFileType;

InputFileType	KnownFile[]=
{	30534, 0x25e0,
	30926, 0x2640
};
map< int, int>   BaseId;

//==========================================================================
int     main( int argc, char *argv[])
{
DbToc   DbSongList("external.db");
char    SqlCmd[256];

BaseId[ 0x03]= 600001;   // Chinese GBK
BaseId[ 0x12]= 700001;   // English
BaseId[ 0x14]= 200001;   // Filipino
BaseId[ 0x04]= 400001;   // Japanese
BaseId[ 0x00]= 500001;   // Korean
BaseId[ 0x16]= 300001;   // Spainish
BaseId[ 0x13]= 800001;   // Vietnamese
BaseId[ 0x05]= 0;        // unknown

char    C, *FName00, *FName20, TxtBuf[256];
unsigned char     LangCode, FileLpd[0x200];
int     I,J, EmptyTitleCnt= 0,
        Mode, KeyLength,
        Len, SongAddr, SongId;
FILE    *Hotdog00, *Hotdog20;

unsigned long songEntryTable[TOC_LENGTH];
unsigned int noSongEntryTables, noSongsTable;
SongEntry *songEntry[TOC_LENGTH];
for( int I= 0; I < TOC_LENGTH; I++)
        songEntry[I]=(SongEntry*)malloc( TOC_LENGTH *sizeof(SongEntry));

//fprintf( stderr,"***");
if( argc!= 2)
{	fprintf( stderr,"Usage: %s <hotdog20>\n", argv[0]);
        return 0;
}
FName20= argv[1];
FName00= strdup( FName20);
FName00[ strlen( FName00)-6]='0';
DbSongList.Sql("INSERT INTO source_info ( name, value) values(\"Product\",\"California Electronics\")");
sprintf( SqlCmd, "INSERT INTO source_info ( name, value) values(\"Path\",\"%s\")",
        FName00);
DbSongList.Sql( SqlCmd);

Hotdog20= fopen( FName20,"rb");
for( I= 0; I< 4; I++)
        SongTotal= (SongTotal << 8)+(int)getc( Hotdog20);

TocOffset= -1;
for( I= 0; I< sizeof( KnownFile)/sizeof( InputFileType); I++)
        if( SongTotal== KnownFile[I].SongTotal)
        {	TocOffset= KnownFile[I].Offset;
                break;
        }
if( TocOffset < 0)
{       fprintf( stderr,"%s unknown file\n", FName20);
        fclose( Hotdog20);
        return 1;
}
Hotdog00= fopen( FName00,"rb");         // Get hotdog00 table info
noSongEntryTables = getTableOfContents( Hotdog00, songEntryTable);
//fprintf( stderr,"%s - %d\n", FName00, noSongEntryTables);
for( int I= 0; I < TOC_LENGTH; I++)
{       noSongsTable= getSongEntryTable( Hotdog00, songEntryTable[I], songEntry[I]);
        fprintf( stderr,"Table %3d: %d\n", I, noSongsTable);
}

printf("%d songs, read from 0x%06x\n", SongTotal, TocOffset);
fseek( Hotdog20, TocOffset , SEEK_SET);

DbSongList.Sql("BEGIN");
for( I= 0; I< SongTotal; I++ )
//for( I= 0; I< 23000; I++ )
{       std::string     Title, Artist;
        unsigned char   TableNo, SongNo;
        Len= getc( Hotdog20);
        LangCode= getc( Hotdog20);
        TableNo= getc( Hotdog20);
        SongNo= getc( Hotdog20);
        for( J= 0; J< Len-4; J++)
        {	C= getc( Hotdog20);
                TxtBuf[J]= C;
        }
        // Only English supported
        if( LangCode!= 0x12)
                continue;

        SongAddr=( TableNo << 8)+SongNo;
        SongId= BaseId[ LangCode]+ SongAddr;
        TxtBuf[J]= 0;
        Title= TxtBuf;
        ReplaceQuote( &Title);
//        printf("%d %02x-%02x %s\n", SongId, TableNo, SongNo, TxtBuf);
        Mode= ASCII_MODE;

//        printf("%6d %s\n", SongId, Title.c_str());
        if( !Title.empty())
        {       KeyLength= ReadLyricDat( Hotdog00, songEntry[ TableNo][ SongNo].offset+8, FileLpd, Mode);
                ReadArtist( FileLpd, TxtBuf, KeyLength, Mode);
                Artist= TxtBuf;
                ReplaceQuote( &Artist);
                sprintf( SqlCmd, "INSERT INTO songs ( id, title, artist, lang_code, addr) values( %d,\"%s\",\"%s\",%d,%d)",
                        SongId, Title.c_str(),Artist.c_str(), LangCode, SongAddr);
                DbSongList.Sql( SqlCmd);
        }
        else
                ++EmptyTitleCnt;
        if( !(I%500))
                fprintf( stderr,"%d\n",I);
}
DbSongList.Sql("END");

fclose( Hotdog00);
fclose( Hotdog20);
for( int I= 0; I < TOC_LENGTH; I++)
        free( songEntry[I]);
free( FName00);
fprintf( stderr,"%d/%d empty title(s) found.\n", EmptyTitleCnt, SongTotal);
return 0;
}


